package com.company.sample.oauth;

import io.jmix.authserver.introspection.AuthorizationServiceOpaqueTokenIntrospector;
import io.jmix.authserver.introspection.TokenIntrospectorRolesHelper;
import io.jmix.authserver.introspection.UserDetailsOAuth2AuthenticatedPrincipal;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.oauth2.core.*;
import org.springframework.security.oauth2.server.authorization.OAuth2Authorization;
import org.springframework.security.oauth2.server.authorization.OAuth2AuthorizationService;
import org.springframework.security.oauth2.server.authorization.OAuth2TokenType;
import org.springframework.security.oauth2.server.resource.introspection.BadOpaqueTokenException;
import org.springframework.stereotype.Component;

import java.security.Principal;
import java.util.ArrayList;
import java.util.Collection;

@Component
public class MyAuthorizationServiceOpaqueTokenIntrospector extends AuthorizationServiceOpaqueTokenIntrospector {

    private final OAuth2AuthorizationService authorizationService;
    private final TokenIntrospectorRolesHelper introspectorRolesHelper;

    public MyAuthorizationServiceOpaqueTokenIntrospector(OAuth2AuthorizationService authorizationService,
                                                         TokenIntrospectorRolesHelper introspectorRolesHelper) {
        super(authorizationService, introspectorRolesHelper);
        this.authorizationService = authorizationService;
        this.introspectorRolesHelper = introspectorRolesHelper;
    }

    @Override
    public OAuth2AuthenticatedPrincipal introspect(String token) {
        OAuth2Authorization authorization = authorizationService.findByToken(token, OAuth2TokenType.ACCESS_TOKEN);
        if (authorization == null) {
            throw new BadOpaqueTokenException("Authorization for provided access token not found");
        }
        OAuth2Authorization.Token<OAuth2Token> accessToken = authorization.getToken(token);
        if (accessToken == null || !accessToken.isActive()) {
            throw new OAuth2AuthenticationException(OAuth2ErrorCodes.INVALID_TOKEN);
        }
        String principalName = authorization.getPrincipalName();
        Collection<GrantedAuthority> authorities = new ArrayList<>();
        if  (AuthorizationGrantType.AUTHORIZATION_CODE.equals(authorization.getAuthorizationGrantType())) {
            Object principal = authorization.getAttribute(Principal.class.getCanonicalName());
            if  (principal instanceof Authentication) {
                principalName = ((Authentication) principal).getName();
                authorities.addAll(((Authentication) principal).getAuthorities());
            }
        } else if  (AuthorizationGrantType.CLIENT_CREDENTIALS.equals(authorization.getAuthorizationGrantType())) {
            principalName = authorization.getPrincipalName();
            try {
                authorities.addAll(introspectorRolesHelper.getClientGrantedAuthorities(principalName));
            } catch (UsernameNotFoundException e) {
                throw new BadOpaqueTokenException("User " + principalName + " not found");
            }
        }
        return new UserDetailsOAuth2AuthenticatedPrincipal(principalName, authorization.getAttributes(), authorities);
    }
}
